
NilSubs := function(L)
    local d, F, subs, A, o, v, w, U, C, hom, img, all;

    # check
    if IsLieNilpotent(L) then return [L]; fi;
    
    # set up
    d := Dimension(L);
    F := LeftActingDomain(L);
    subs := [];

    # get autos and orbs
    #A := AutomorphismGroup(L);
    #o := MatOrbs( GeneratorsOfGroup(A), d, F );
    o := Elements(F^d);
    o := Filtered(o, x -> x <> 0*x);

    # loop
    for v in o do 
        w := v * Basis(L);
        U := Subalgebra(L, [w]);
        C := LieCentralizer(L,U);
        
        if IsLieNilpotent(C) then 
            Add(subs, C);
        else
            hom := NaturalHomomorphismByIdeal(C,U);
            img := NilSubs( Image(hom) );
            all := List(img, x -> PreImage(hom, x));
            Append(subs, all);
        fi;
    od;

    return subs;
end;

CanoBasis := function( B, U )
    local b;
    b := List(Basis(U), x -> Coefficients(B,x));
    return TriangulizedMat(b);
end;

MyOrbitReps := function( A, L, subs )
    local B, base, todo, reps, i, o, j;
    B := Basis(L);
    base := List(subs, x -> CanoBasis(B,x));
    todo := List(subs, x -> true);
    reps := [];
    while ForAny(todo, x -> x = true) do
        i := Position(todo, true);
        todo[i] := false;
        Add(reps, subs[i]);
        o := Orbit(A, base[i], OnBases);
        for j in [1..Length(base)] do
            if todo[j]=true and base[j] in o then 
                todo[j] := false;
            fi;
        od;
    od;
    return reps;
end;

CSAS := function(L)
    local d, F, subs, A, o, v, w, U, C, hom, img, all, H, N;

    # check
    if IsLieNilpotent(L) then return [L]; fi;
    
    # set up
    d := Dimension(L);
    F := LeftActingDomain(L);
    subs := [];

    # get autos and orbs
    Print("CSA: determine aut group \n");
    A := AutomorphismGroup(L);
    o := MatOrbs( GeneratorsOfGroup(A), d, F );

    # loop
    Print("CSA: loop over ",Length(o)," elements \n");
    for v in o do 
        w := v * Basis(L);
        U := Subalgebra(L, [w]);
        C := LieCentralizer(L,U);
        
        if IsLieNilpotent(C) then 
            all := [C];
        else
            hom := NaturalHomomorphismByIdeal(C,U);
            img := NilSubs( Image(hom) );
            all := List(img, x -> PreImage(hom, x));
        fi;

        for H in all do
            N := LieNormalizer(L,H);
            if Dimension(N) = Dimension(H) then Add(subs, H); fi;
        od;
        Print("CSA: element ",Position(o,v)," yields ",Length(subs),"\n");
    od;

    # reduce
    Print("CSA: reduce to orbit reps \n");
    subs := MyOrbitReps(A, L, subs);

    L!.csas := subs;
    return subs;
end;

EngelSubalgebra := function( L, x )
    local m, r, i, k;

    m := TransposedMat(AdjointMatrix( Basis(L), x*Basis(L) ));
    r := Length(NullspaceMat(m));
    i := 1;
    repeat
        i := i+1;
        k := NullspaceMat(m^i);
        if Length(k) = r then 
            return Subalgebra(L, k * Basis(L));
        fi;
        r := Length(k);
    until false;
end;

EngelSubalgebras := function(L)
    local d, F, A, o, E, i, N;

    # get elements
    d := Dimension(L);
    F := LeftActingDomain(L);
    A := AutomorphismGroup(L);
    o := MatOrbs( GeneratorsOfGroup(A), d, F );

    # get subspaces
    E := List(o, x -> EngelSubalgebra(L,x));

    # investigate
    N := List(E, x -> LieNormalizer(L,x));

    # return
    return rec( elms := o, engel := E, norms := N);

    # investigate
    for i in [1..Length(E)] do
        if IsLieNilpotent(E[i]) then 
            N := LieNormalizer(L,E[i]);
            if Dimension(N) = Dimension(E[i]) then 
                Print(i," yields CSA with ",Dimension(E[i]),"\n");
            fi;
        fi;
    od;

end;

EngelCSAS := function(L)
    local engel, i;
    engel := EngelSubalgebras(L);
    for i in [1..Length(engel.engel)] do
        if Dimension(engel.engel[i]) <> Dimension(engel.norms[i]) then
            Error("wrong engel");
        fi;
    od;
    return Filtered(engel.engel, IsLieNilpotent);
end;

GenericCharPoly := function(L)
    local n, F, B, x, i, t, T, M, N, j, k, f, c, s;

    # set up
    n := Dimension(L);
    F := LeftActingDomain(L);
    B := Basis(L);

    # get indets
    x := List([1..n], i -> Indeterminate(F,i));
    t := Indeterminate(F,n+1); 

    # structure constants
    T := List([1..n], j -> List([1..n], i -> Coefficients(B, B[j]*B[i])));

    # get matrix
    M := List([1..n], j -> List([1..n], k -> false));
    for j in [1..n] do
        for k in [1..n] do
            M[j][k] := Sum(List([1..n], i -> T[j][i][k]*x[i]));
        od;
    od;
    N := M - t*IdentityMat(n,F);

    # get char poly
    f := DeterminantMatDivFree(N);
    c := PolynomialCoefficientsOfPolynomial(f,t);
    s := First([1..Length(c)], x -> c[x] <> 0*c[x]);

    return rec(deg := s, pol := c[s]);
end;

RegularElements := function(L)
    local o, A, n, F, x, c, f, v, w, elms;

    # set up
    n := Dimension(L);
    F := LeftActingDomain(L);
    A := AutomorphismGroup(L);
    o := MatOrbs( GeneratorsOfGroup(A), n, F );
    Print("got ",Length(o), " orbits \n");

    # get poly
    x := List([1..n], i -> Indeterminate(F, i));
    c := GenericCharPoly(L);
    f := c.pol;
    Print("got poly in deg ",c.deg, " of type \n");
    Print(f,"\n");

    # loop
    elms := [];
    for v in o do
        w := Value( f, x, v );
        if w <> 0*w then Add(elms, v); fi;
    od;
    return elms;
end;

EngelCSASReg := function(L)
    local elm, i, U, N;
    elm := RegularElements(L);
    Print("found ",Length(elm)," regular elements \n");
    for i in [1..Length(elm)] do
        U := EngelSubalgebra(L, elm[i]);
        N := LieNormalizer(L,U);
        if Dimension(U) = Dimension(N) and IsLieNilpotent(U) then 
            Print("found csa of dim ",Dimension(U)," \n");
        else
            Print("failure \n");
        fi;
    od;
end;

LocalEngelCSAS := function(L)
    local n, F, A, o, deg, eng, i, m, f, c, s, U, t, res;

    # set up
    n := Dimension(L);
    F := LeftActingDomain(L);
    A := AutomorphismGroup(L);
    o := MatOrbs( GeneratorsOfGroup(A), n, F );
    t := Indeterminate(F);
    Print("got ",Length(o), " orbits \n");

    res := [];
    for i in [1..Length(o)] do
        m := TransposedMat(AdjointMatrix(Basis(L), o[i]*Basis(L)));
        f := Determinant( m-t*IdentityMat(n,F));
        c := PolynomialCoefficientsOfPolynomial(f, t);
        s := First([1..Length(c)], x -> c[x] <> 0*c[x]);
        U := EngelSubalgebra(L, o[i]);
        if IsLieNilpotent(U) then 
            Add(res, [s, Dimension(U)]);
            Print(i, " yields deg ",s," of dim ", Dimension(U),"\n");
        fi;
    od;
    return Collected(res);
end;

