
ProductTensor := function( e, x, y )
    local z, i, j;
    z := List(e, x -> 0);
    for i in [1..Length(x)] do
        for j in [1..Length(y)] do
            z[Position(e,[i,j])] := x[i]*y[j];
        od;
    od;
    return z;
end;

InstallGlobalFunction( LieAlgebraTensor, function(L, K)
    local B, b, F, R, C, c, n, T, e, i, j, l, k, r, s, x, y, z, V;

    # set up L
    B := Basis(L);
    b := BasisVectors(B);
    F := LeftActingDomain(L);

    # set up field extension
    R := AsField(F,K);
    C := Basis(R);
    c := BasisVectors(C);

    # structure constants for tensor
    n := Length(b)*Length(c);
    T := EmptySCTable( n, Zero(F), "antisymmetric" );
    e := Cartesian([1..Length(b)],[1..Length(c)]);

    # fill in structure constants
    for i in [1..Length(b)] do
        for j in [1..Length(c)] do
            for l in [1..Length(b)] do
                for k in [1..Length(c)] do
                    r := Position(e,[i,j]);
                    s := Position(e,[l,k]);
                    if r < s then 
                        x := Coefficients( B, b[i]*b[l] );
                        y := Coefficients( C, c[j]*c[k] );
                        z := ProductTensor( e, x, y ) * One(F);
                        SetEntrySCTable( T, r, s, WordOfVector(z) );
                    fi;
                od;
            od;
        od;
    od;

    # construct algebra
    V := LieAlgebraByStructureConstants( F, T );

    # add base
    V!.base := e;

    # that's it
    return V;
end );


