
ExceptionalLieAlgebrasGF2 := function(limit)
    local all, par, parQ, parP, n, e, m;

    all := [];
    par := Concatenation( List( [2..LogInt(limit,2)+1], Partitions ) );


    ###################################################################
    # Vaughan-Lee

    if limit>=8 then Add(all, VaughanLeeAlgebras(4)); fi;

    ###################################################################
    # Lin Lei and Zhang Yongzheng

    parQ := Filtered(par, x -> not IsInt(Length(x)/2) and Length(x)>1);
    for n in parQ do
        for e in GF(2)^((Length(n)-1)/2) do
            Add(all, SimpleLieAlgebraTypeK2(n, e, GF(2)));
        od;
    od;

    ###################################################################
    # Lin Lei

    # type P and n = 1
    parP := Filtered(par, x -> Set(x) = [1] and Length(x)>3);
    for n in parP do
         Add(all, SimpleLieAlgebraTypeH2(n, GF(2))); 
    od;

    # type P and n <> 1
    parP := Filtered(par, x -> Set(x) <> [1]);
    for n in parP do 
        Add(all, SimpleLieAlgebraTypeH2(n, GF(2))); 
    od;

    ###################################################################
    # Kaplanski 

    m := LogInt(limit-2,2);
    for n in [4..m] do
        Add(all, SimpleLieAlgebraByGramMatrix1(n));
    od;

    m := LogInt(limit-1,2);
    for n in Filtered([2..m], x -> IsInt(x/2)) do
        Add(all, SimpleLieAlgebraByGramMatrix2(n));
    od;

    m := RootInt(2*limit+1,2)+2;
    for n in [5..m] do
        if n*(n-1)/2 <= limit then 
            Add(all, SimpleLieAlgebraAlternateMats(n));
        fi;
    od;

    if limit >= 2016 then 
        Print("Kaplanski's case 4 is missing!");
    fi;

    ###################################################################
    # Brown 

    m := LogInt(limit+2,2)-2;
    par := Concatenation( List( [2..m], x -> Partitions(x,2) ) );
    for n in par do 
        Add(all, SimpleLieAlgebraTypeBR2(n,GF(2))); 
    od;

    m := LogInt(limit+6,2)-2;
    par := Concatenation( List( [2..m], x -> Partitions(x,3) ) );
    for n in par do 
        Add(all, SimpleLieAlgebraTypeBR1(n,GF(2))); 
    od;

    ###################################################################
    # Fehlt noch: Variations of G2. F"ur diese scheint das Paper schwer
    # zu bekommen. Die Lie algs haben Dimensionen 8, 14, 15. 

    ###################################################################
    # final 
    all := Filtered(all, x -> Dimension(x) <= limit);
    Sort( all, function(a,b) return Dimension(a)<Dimension(b); end );
    return all;
end;

KnownCSimpleLieAlgebras := function( F, limit )
    local a, b, c, d, i, r, p, L;
    p := Characteristic(F);
    a := ClassicSimpleLieAlgebras( F, limit );
    b := ModularSimpleLieAlgebras( F, limit );
    if Characteristic(F) = 2 then 
        c := ExceptionalLieAlgebrasGF2( limit );
    else
        c := [];
    fi;
    a := Concatenation(a,b,c);
    Sort( a, function(a,b) return Dimension(a)<Dimension(b); end );
    return a;
end;

KnownTensorLieAlgebrasDB := function( F, limit )
    local a, i, j, r, p, L, M, SL, ST;
    p := Characteristic(F);
    if p = 2 then SL := SLDB2; elif p = 3 then SL := SLDB3; fi;
    if p = 2 then ST := SLTB2; elif p = 3 then ST := SLTB3; fi;
    a := [];
    for i in [1..Length(ST)] do
        for j in [1..ST[i]] do
            M := SL[i][j];
            if not 'x' in M!.names[1] then 
                r := 2;
                while i*r <= limit do
                    L := LieAlgebraTensor(M, GF(p^r));
                    SetName(L, Concatenation(M!.names[1],"x", String(p^r)));
                    L!.names := [L!.name]; 
                    Add(a, L);
                    r := r+1;
                od;
            fi;
        od;
    od;
    Sort( a, function(a,b) return Dimension(a)<Dimension(b); end );
    return a;
end;

